@@ -403,8 +403,8 @@ open_converter (const gchar *to_codeset,
 		 from_codeset, to_codeset);
   else
     g_set_error (error, G_CONVERT_ERROR, G_CONVERT_ERROR_FAILED,
-		 _("Could not open converter from '%s' to '%s': %s"),
-		 from_codeset, to_codeset, g_strerror (errno));
+		 _("Could not open converter from '%s' to '%s'"),
+		 from_codeset, to_codeset);
   
   return cd;
 }
@@ -1088,7 +1088,12 @@ get_filename_charset (const gchar **filename_charset)
 }
 
 #else /* G_PLATFORM_WIN32 */
-#define get_filename_charset (charset) TRUE
+static gboolean
+get_filename_charset (const gchar **filename_charset) 
+{
+  g_get_charset (filename_charset);
+  return FALSE;
+}
 #endif /* G_PLATFORM_WIN32 */
 
 /* This is called from g_thread_init(). It's used to
