@@ -48,6 +48,14 @@
 #  include <sys/poll.h>
 #  undef events	 /* AIX 4.1.5 & 4.3.2 define this for SVR3,4 compatibility */
 #  undef revents /* AIX 4.1.5 & 4.3.2 define this for SVR3,4 compatibility */
+
+/* The poll() emulation on OS/X doesn't handle fds=NULL, nfds=0,
+ * so we prefer our own poll emulation.
+ */
+#ifdef _POLL_EMUL_H_
+#undef HAVE_POLL
+#endif
+   
 #endif /* GLIB_HAVE_SYS_POLL_H */
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
@@ -194,6 +202,8 @@ struct _GPollRec
 #endif
 
 #define SOURCE_DESTROYED(source) (((source)->flags & G_HOOK_FLAG_ACTIVE) == 0)
+#define SOURCE_BLOCKED(source) (((source)->flags & G_HOOK_FLAG_IN_CALL) != 0 && \
+		                ((source)->flags & G_SOURCE_CAN_RECURSE) == 0)
 
 #define SOURCE_UNREF(source, context)                       \
    G_STMT_START {                                           \
@@ -261,9 +271,6 @@ static gint child_watch_wake_up_pipe[2] = {0, 0};
 G_LOCK_DEFINE_STATIC (main_context_list);
 static GSList *main_context_list = NULL;
 
-#if defined(G_PLATFORM_WIN32) && defined(__GNUC__)
-__declspec(dllexport)
-#endif
 GSourceFuncs g_timeout_funcs =
 {
   g_timeout_prepare,
@@ -280,9 +287,6 @@ GSourceFuncs g_child_watch_funcs =
   NULL
 };
 
-#if defined(G_PLATFORM_WIN32) && defined(__GNUC__)
-__declspec(dllexport)
-#endif
 GSourceFuncs g_idle_funcs =
 {
   g_idle_prepare,
@@ -966,14 +970,17 @@ g_source_destroy_internal (GSource      *source,
 	  old_cb_funcs->unref (old_cb_data);
 	  LOCK_CONTEXT (context);
 	}
-      
-      tmp_list = source->poll_fds;
-      while (tmp_list)
+
+      if (!SOURCE_BLOCKED (source))
 	{
-	  g_main_context_remove_poll_unlocked (context, tmp_list->data);
-	  tmp_list = tmp_list->next;
+	  tmp_list = source->poll_fds;
+	  while (tmp_list)
+	    {
+	      g_main_context_remove_poll_unlocked (context, tmp_list->data);
+	      tmp_list = tmp_list->next;
+	    }
 	}
-      
+	  
       g_source_unref_internal (source, context, TRUE);
     }
 
@@ -1079,7 +1086,8 @@ g_source_add_poll (GSource *source,
 
   if (context)
     {
-      g_main_context_add_poll_unlocked (context, source->priority, fd);
+      if (!SOURCE_BLOCKED (source))
+	g_main_context_add_poll_unlocked (context, source->priority, fd);
       UNLOCK_CONTEXT (context);
     }
 }
@@ -1111,7 +1119,8 @@ g_source_remove_poll (GSource *source,
 
   if (context)
     {
-      g_main_context_remove_poll_unlocked (context, fd);
+      if (!SOURCE_BLOCKED (source))
+	g_main_context_remove_poll_unlocked (context, fd);
       UNLOCK_CONTEXT (context);
     }
 }
@@ -1265,16 +1274,22 @@ g_source_set_priority (GSource  *source,
 
   if (context)
     {
-      source->next = NULL;
-      source->prev = NULL;
-      
-      tmp_list = source->poll_fds;
-      while (tmp_list)
+      /* Remove the source from the context's source and then
+       * add it back so it is sorted in the correct plcae
+       */
+      g_source_list_remove (source, source->context);
+      g_source_list_add (source, source->context);
+
+      if (!SOURCE_BLOCKED (source))
 	{
-	  g_main_context_remove_poll_unlocked (context, tmp_list->data);
-	  g_main_context_add_poll_unlocked (context, priority, tmp_list->data);
-      
-	  tmp_list = tmp_list->next;
+	  tmp_list = source->poll_fds;
+	  while (tmp_list)
+	    {
+	      g_main_context_remove_poll_unlocked (context, tmp_list->data);
+	      g_main_context_add_poll_unlocked (context, priority, tmp_list->data);
+	      
+	      tmp_list = tmp_list->next;
+	    }
 	}
       
       UNLOCK_CONTEXT (source->context);
@@ -1790,7 +1805,7 @@ get_depth_pointer (void)
  *     {
  *       GList *next = l->next;
  *       FreeListBlock *block = l->data;
- *       if (block->depth > depth);
+ *       if (block->depth > depth)
  *         {
  *           g_free (block->mem);
  *           g_free (block);
@@ -1839,6 +1854,43 @@ g_main_depth (void)
   return *depth;
 }
 
+/* Temporarily remove all this source's file descriptors from the
+ * poll(), so that if data comes available for one of the file descriptors
+ * we don't continually spin in the poll()
+ */
+/* HOLDS: source->context's lock */
+void
+block_source (GSource *source)
+{
+  GSList *tmp_list;
+
+  g_return_if_fail (!SOURCE_BLOCKED (source));
+
+  tmp_list = source->poll_fds;
+  while (tmp_list)
+    {
+      g_main_context_remove_poll_unlocked (source->context, tmp_list->data);
+      tmp_list = tmp_list->next;
+    }
+}
+
+/* HOLDS: source->context's lock */
+void
+unblock_source (GSource *source)
+{
+  GSList *tmp_list;
+  
+  g_return_if_fail (!SOURCE_BLOCKED (source)); /* Source already unblocked */
+  g_return_if_fail (!SOURCE_DESTROYED (source));
+  
+  tmp_list = source->poll_fds;
+  while (tmp_list)
+    {
+      g_main_context_add_poll_unlocked (source->context, source->priority, tmp_list->data);
+      tmp_list = tmp_list->next;
+    }
+}
+
 /* HOLDS: context's lock */
 static void
 g_main_dispatch (GMainContext *context)
@@ -1875,6 +1927,9 @@ g_main_dispatch (GMainContext *context)
 	  if (cb_funcs)
 	    cb_funcs->ref (cb_data);
 	  
+	  if ((source->flags & G_SOURCE_CAN_RECURSE) == 0)
+	    block_source (source);
+	  
 	  was_in_call = source->flags & G_HOOK_FLAG_IN_CALL;
 	  source->flags |= G_HOOK_FLAG_IN_CALL;
 
@@ -1897,6 +1952,10 @@ g_main_dispatch (GMainContext *context)
 	 if (!was_in_call)
 	    source->flags &= ~G_HOOK_FLAG_IN_CALL;
 
+	  if ((source->flags & G_SOURCE_CAN_RECURSE) == 0 &&
+	      !SOURCE_DESTROYED (source))
+	    unblock_source (source);
+	  
 	  /* Note: this depends on the fact that we can't switch
 	   * sources from one main context to another
 	   */
@@ -2185,7 +2244,7 @@ g_main_context_prepare (GMainContext *context,
 	  SOURCE_UNREF (source, context);
 	  break;
 	}
-      if ((source->flags & G_HOOK_FLAG_IN_CALL) && !(source->flags & G_SOURCE_CAN_RECURSE))
+      if (SOURCE_BLOCKED (source))
 	goto next;
 
       if (!(source->flags & G_SOURCE_READY))
@@ -2290,7 +2349,7 @@ g_main_context_query (GMainContext *context,
   if (timeout)
     {
       *timeout = context->timeout;
-      if (timeout != 0)
+      if (*timeout != 0)
 	context->time_is_current = FALSE;
     }
   
@@ -2373,7 +2432,7 @@ g_main_context_check (GMainContext *context,
 	  SOURCE_UNREF (source, context);
 	  break;
 	}
-      if ((source->flags & G_HOOK_FLAG_IN_CALL) && !(source->flags & G_SOURCE_CAN_RECURSE))
+      if (SOURCE_BLOCKED (source))
 	goto next;
 
       if (!(source->flags & G_SOURCE_READY))
@@ -3601,7 +3660,8 @@ g_child_watch_source_init (void)
 
 /**
  * g_child_watch_source_new:
- * @pid: process id of a child process to watch
+ * @pid: process id of a child process to watch. On Windows, a HANDLE
+ * for the process to watch (which actually doesn't have to be a child).
  * 
  * Creates a new child_watch source.
  *
