@@ -585,9 +585,31 @@ g_path_get_dirname (const gchar	   *file_name)
   }
 #endif
   if (!base)
+    {
+#ifdef G_OS_WIN32
+      if (g_ascii_isalpha (file_name[0]) && file_name[1] == ':')
+	{
+	  gchar drive_colon_dot[4];
+
+	  drive_colon_dot[0] = file_name[0];
+	  drive_colon_dot[1] = ':';
+	  drive_colon_dot[2] = '.';
+	  drive_colon_dot[3] = '\0';
+
+	  return g_strdup (drive_colon_dot);
+	}
+#endif
     return g_strdup (".");
+    }
+
   while (base > file_name && G_IS_DIR_SEPARATOR (*base))
     base--;
+
+#ifdef G_OS_WIN32
+  if (base == file_name + 1 && g_ascii_isalpha (file_name[0]) && file_name[1] == ':')
+      base++;
+#endif
+
   len = (guint) 1 + base - file_name;
   
   base = g_new (gchar, len + 1);
@@ -959,7 +981,7 @@ g_get_any_init (void)
 	  {
  	    g_user_name = g_strdup (pw->pw_name);
 
-	    if (pw->pw_gecos) 
+	    if (pw->pw_gecos && *pw->pw_gecos != '\0') 
 	      {
 		gchar **gecos_fields;
 		gchar **name_parts;
